@@ -1,12 +1,22 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2017 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the examples of the Qt Wayland module
 **
 ** $QT_BEGIN_LICENSE:BSD$
-** You may use this file under the terms of the BSD license as follows:
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
+**
+** BSD License Usage
+** Alternatively, you may use this file under the terms of the BSD license
+** as follows:
 **
 ** "Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that the following conditions are
@@ -47,6 +57,7 @@
 #include <QtWaylandCompositor/QWaylandWlShellSurface>
 #include <QtWaylandCompositor/QWaylandXdgSurfaceV5>
 #include <QTimer>
+#include <QOpenGLTextureBlitter>
 
 QT_BEGIN_NAMESPACE
 
@@ -54,33 +65,51 @@ class QWaylandWlShell;
 class QWaylandWlShellSurface;
 class QWaylandXdgShellV5;
 class QOpenGLTexture;
+class Compositor;
 
 class View : public QWaylandView
 {
     Q_OBJECT
 public:
-    View();
+    View(Compositor *compositor);
     QOpenGLTexture *getTexture();
+    QOpenGLTextureBlitter::Origin textureOrigin() const;
     QPointF position() const { return m_position; }
     void setPosition(const QPointF &pos) { m_position = pos; }
+    QSize size() const;
     bool isCursor() const;
     bool hasShell() const { return m_wlShellSurface; }
     void setParentView(View *parent) { m_parentView = parent; }
     View *parentView() const { return m_parentView; }
     QPointF parentPosition() const { return m_parentView ? (m_parentView->position() + m_parentView->parentPosition()) : QPointF(); }
-    QSize windowSize() { return m_xdgSurface ? m_xdgSurface->windowGeometry().size() : surface()->size(); }
+    QSize windowSize() { return m_xdgSurface ? m_xdgSurface->windowGeometry().size() : surface() ? surface()->destinationSize() : m_size; }
     QPoint offset() const { return m_offset; }
 
+    qreal animationFactor() const {return m_animationFactor; }
+    void setAnimationFactor(qreal f) {m_animationFactor = f; }
+
+signals:
+    void animationDone();
+
+protected:
+    void timerEvent(QTimerEvent *event) override;
+
 private:
     friend class Compositor;
-    GLenum m_textureTarget;
-    QOpenGLTexture *m_texture;
+    Compositor *m_compositor = nullptr;
+    GLenum m_textureTarget = GL_TEXTURE_2D;
+    QOpenGLTexture *m_texture = nullptr;
+    QOpenGLTextureBlitter::Origin m_origin;
     QPointF m_position;
-    QWaylandWlShellSurface *m_wlShellSurface;
-    QWaylandXdgSurfaceV5 *m_xdgSurface;
-    QWaylandXdgPopupV5 *m_xdgPopup;
-    View *m_parentView;
+    QSize m_size;
+    QWaylandWlShellSurface *m_wlShellSurface = nullptr;
+    QWaylandXdgSurfaceV5 *m_xdgSurface = nullptr;
+    QWaylandXdgPopupV5 *m_xdgPopup = nullptr;
+    View *m_parentView = nullptr;
     QPoint m_offset;
+    qreal m_animationFactor = 1.0;
+    QBasicTimer m_animationTimer;
+    bool m_animationCountUp;
 
 public slots:
     void onXdgSetMaximized();
@@ -88,6 +117,9 @@ public slots:
     void onXdgSetFullscreen(QWaylandOutput *output);
     void onXdgUnsetFullscreen();
     void onOffsetForNextFrame(const QPoint &offset);
+
+    void startAnimation(bool countUp);
+    void cancelAnimation();
 };
 
 class Compositor : public QWaylandCompositor
@@ -95,8 +127,8 @@ class Compositor : public QWaylandCompositor
     Q_OBJECT
 public:
     Compositor(QWindow *window);
-    ~Compositor();
-    void create() Q_DECL_OVERRIDE;
+    ~Compositor() override;
+    void create() override;
 
     void startRender();
     void endRender();
@@ -119,6 +151,9 @@ signals:
     void dragStarted(View *dragIcon);
     void frameOffset(const QPoint &offset);
 
+public slots:
+    void triggerRender();
+
 private slots:
     void surfaceHasContentChanged();
     void surfaceDestroyed();
@@ -129,7 +164,6 @@ private slots:
 
     void startDrag();
 
-    void triggerRender();
 
     void onSurfaceCreated(QWaylandSurface *surface);
     void onWlShellSurfaceCreated(QWaylandWlShellSurface *wlShellSurface);
@@ -143,12 +177,13 @@ private slots:
     void onSubsurfacePositionChanged(const QPoint &position);
 
     void updateCursor();
+    void viewAnimationDone();
 private:
     View *findView(const QWaylandSurface *s) const;
-    QWindow *m_window;
+    QWindow *m_window = nullptr;
     QList<View*> m_views;
-    QWaylandWlShell *m_wlShell;
-    QWaylandXdgShellV5 *m_xdgShell;
+    QWaylandWlShell *m_wlShell = nullptr;
+    QWaylandXdgShellV5 *m_xdgShell = nullptr;
     QWaylandView m_cursorView;
     int m_cursorHotspotX;
     int m_cursorHotspotY;
